"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _koa = _interopRequireDefault(require("koa"));

var _nodeToolkit = require("@tech_query/node-toolkit");

var _koaLogger = _interopRequireDefault(require("koa-logger"));

var _koaBody = _interopRequireDefault(require("koa-body"));

var _ProxyAgent = _interopRequireDefault(require("./ProxyAgent"));

var _cors = _interopRequireDefault(require("@koa/cors"));

var _koaStatic = _interopRequireDefault(require("koa-static"));

var _internalIp = _interopRequireDefault(require("internal-ip"));

var _opn = _interopRequireDefault(require("opn"));

var _url = require("url");

var _path = require("path");

var _child_process = require("child_process");

/**
 * Web server
 */
class WebServer {
  /**
   * @param {string}         [staticPath='.']
   * @param {number}         [netPort=0]
   * @param {boolean}        [XDomain=false]
   * @param {?Object}        proxyMap         - Same as the parameter of {@link patternOf}
   * @param {boolean|string} [openURL=false]
   */
  constructor(staticPath, netPort, XDomain, proxyMap, openURL) {
    /**
     * @type {string}
     */
    this.staticPath = staticPath || '.';
    /**
     * @type {number}
     */

    this.netPort = +(netPort || 0);
    /**
     * @private
     *
     * @type {Application}
     */

    this.core = new _koa.default();
    /**
     * @type {boolean}
     */

    this.XDomain = XDomain;
    /**
     * @private
     *
     * @type {?Object}
     */

    this.proxyMap = (0, _nodeToolkit.patternOf)(proxyMap);
    /**
     * @private
     *
     * @type {boolean|string}
     */

    this.openPath = openURL;
    /**
     * @private
     *
     * @type {ServerAddress}
     */

    this.address = null;
    this.boot();
  }
  /**
   * @private
   */


  boot() {
    this.core.use((0, _koaLogger.default)());
    if (this.proxyMap) this.core.use((0, _koaBody.default)({
      multipart: true
    })).use((0, _ProxyAgent.default)(this.proxyMap));
    if (this.XDomain) this.core.use((0, _cors.default)());
    this.core.use((0, _koaStatic.default)(this.staticPath));
  }
  /**
   * Origin URI
   *
   * @type {string}
   */


  get URL() {
    var address = this.address;
    return address ? `http://${address.address}:${address.port}` : '';
  }
  /**
   * URL to open in default browser
   *
   * @type {string}
   */


  get openURL() {
    return typeof this.openPath !== 'string' ? this.URL : (0, _url.resolve)(this.URL, this.openPath);
  }
  /**
   * Create a server in the same Node.JS process
   *
   * @return {Server} HTTP server
   */


  localHost() {
    var server = this;
    return this.core.listen(this.netPort,
    /*#__PURE__*/
    (0, _asyncToGenerator2.default)(
    /*#__PURE__*/
    _regenerator.default.mark(function _callee() {
      return _regenerator.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              server.address = Object.assign(this.address(), {
                family: 'IPv4',
                address: _internalIp.default.v4.sync()
              });

              if (!process.send) {
                _context.next = 3;
                break;
              }

              return _context.abrupt("return", process.send({
                type: 'ready',
                data: server.address
              }));

            case 3:
              console.info(`Web server runs at ${server.URL}`);

              if (!server.openPath) {
                _context.next = 7;
                break;
              }

              _context.next = 7;
              return (0, _opn.default)(server.openURL);

            case 7:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }))).on('error', function (error) {
      if (process.send) process.send({
        type: 'error',
        data: error
      });else console.error(error);
    });
  }
  /**
   * Boot a server in a forked Node.JS process
   *
   * @return {Promise<ServerAddress>}
   */


  workerHost() {
    var child = (0, _child_process.fork)((0, _path.join)((0, _nodeToolkit.currentModulePath)(), '../../dist/'), [this.staticPath, '-p', this.netPort, this.XDomain && '--CORS'], {
      execArgv: []
    });
    return new Promise(function (resolve, reject) {
      child.on('message', function (event) {
        switch (event.type) {
          case 'ready':
            return resolve(event.data);

          case 'error':
            {
              var error = event.data;
              reject(Object.assign(new global[error.name](error.message), error));
            }
        }
      });
    });
  }

}
/**
 * @typedef {Object} ServerAddress
 *
 * @property {string} family  - `IPv4`
 * @property {string} address - IP address
 * @property {number} port    - Network listening port
 */

/**
 * @external {Application} https://github.com/koajs/koa/blob/master/docs/api/index.md#application
 */


exports.default = WebServer;
//# sourceMappingURL=WebServer.js.map